#!/usr/local/gnu/bin/bash
#
# latex.sh <Complete> <Basename> <Backupdir> <Bibtex>
#
#    Complete  = 0/1 (kompletter Lauf oder Schnellansicht)
#    Basename  = Name der Haupt-TeX-Quelle (ohne .tex)
#    Backupdir = Name des Backupverz. (ohne abschliessenden /)
#    Bibtex    = 0/1 (BibTeX ausfuehren)
#
#  Autor:
#     Carsten Wilhelm, 11.12.1997
#
#  Methode:
#    * bei fehlendem .aux File oder Aenderungen der citations
#      darin wird bei Complete=1 ein kompletter BibTeX-Lauf
#      durchgefuehrt.
#    * ansonsten wird bei Complete=1 solange LaTeX ausgefuehrt,
#      wie noetig.
#    * nach dem ersten LaTeX-Durchlauf werden bei unter-
#      schiedlichen \citation-Verweisen nochmals LaTeX und
#      BibTeX angeworfen.
#    * bei Complete=0 wird nur einmal LaTeX ausgefuehrt.
#
#  Abhaengigkeiten von den .aux-Files und den .bib-Files
#     werden innerhalb dieses Skriptes behandelt, muessen
#     also NICHT mit ins Makefile uebernommen werden :-)
#

LATEX_MESSAGE="Starte LaTeX"

#
#
abort() {
    if [ $1 -ne 0 ]; then
        echo $2
        exit $1
    fi
}

# Initial LaTeX und BibTeX aufrufen, vorher
#    unnoetige Dateien wegwerfen
bibtex-macro() {
    # Pruefen, ob BibTeX gewollt ist oder BibTeX-Quellen vorhanden sind
    if [ $COMPLETE_RUN -eq 0 -o $RUN_BIBTEX -eq 0 ]; then
        echo "Uebergehe BibTeX (kein kompletter Lauf oder kein BibTeX erwuenscht) ..."
        return 1;
    fi

    # Anzahl der Argumente pruefen
    if [ $# -ne 1 ]; then
        abort 1 "no argument for bibtex-macro()"
    fi

    # Dateien loeschen und LaTeX/BibTeX starten
    rm -f *.aux *.bbl *.blg $BACKUPDIR/$1.aux
    echo Starte LaTeX ...
    latex $1
    abort $? "LaTeX error ..."

    LATEX_MESSAGE="AUX-Files unterschiedlich, starte LaTeX nochmals ..."
    echo Starte BibTeX ...
    bibtex $1
    abort $? "BibTeX error ..."

    return 0
}

diffit-bibtex() {
    # BibTeX nicht gewollt
    if [ $COMPLETE_RUN -eq 0 -o $RUN_BIBTEX -eq 0 ]; then return 1; fi

    # DVI-Datei ist aelter als ein Literaturverzeichnis ==> BibTeX noetig
    for bibfile in $BIBTEX_SRC ; do
        echo Teste BibTeX-Quelle: $bibfile
        if [ $1.dvi -ot $bibfile ]; then return 0 ; fi
    done
    
    # Keine BibTeX-Ausgabe gefunden ==> BibTeX noetig
    if [ ! -f $1.bbl ]; then return 0 ; fi

    # Kein .aux-File gefunden ==> BibTeX noetig
    if [ ! -f $1.aux -o ! -f $BACKUPDIR/$1.aux ]; then return 0 ; fi

    # Vergleich der .aux-Files ergibt Unterschiede ==> BibTeX noetig
    cat *.aux | grep citation | sort -u >$1.aux.tmp
    if [ -n "`diff $1.aux.tmp $BACKUPDIR/$1.aux.bib 2>/dev/null`" ]; then return 0 ; fi
    rm $1.aux.tmp

    # Ansonsten ist kein BibTeX noetig
    return 1
}

# DIFF-Macro, prueft das AUX-File, ob nochmals LaTeX laufen muss
diffit-latex() {
    # LaTeX erzwingen
    if [ $FORCE_LATEX -eq 1 ]; then return 0; fi

    # ohne kompletten Lauf LaTeX verbieten
    if [ $COMPLETE_RUN -eq 0 ]; then return 1; fi

    # kein aux-File gefunden ==> LaTeX noetig
    if [ ! -f $1.aux ]; then return 0 ; fi

    # aux-File enthaelt nur \relax ==> LaTeX unnoetig
    if [ -z "`grep -v relax $1.aux`" ]; then return 1 ; fi

    # kein gesichertes aux-File gefunden ==> LaTeX noetig
    if [ ! -f $BACKUPDIR/$1.aux ]; then return 0 ; fi

    # Unterschiede zwischen aux-Files (aktuell und gesichert) ==> LaTeX noetig
    cat *.aux | sort -u > $1.aux.tmp
    if [ -n "`diff $1.aux.tmp $BACKUPDIR/$1.aux 2>/dev/null`" ]; then return 0 ; fi
    rm $1.aux.tmp

    # ansonsten ist LaTeX nicht noetig
    return 1;
}

# LaTeX genuegend oft aufrufen
#    latex-macro <force-first> <basename>
latex-macro() {
    # Anzahl der Argumente pruefen
    if [ $# -ne 2 ]; then
        echo "no arguments for latex-macro()"
        exit 1
    fi
    FORCE_LATEX=$1
    FIRST_LATEX=$1

    # LaTeX wiederholt aufrufen
    while diffit-latex $2 ; do
        # Zustand VOR LaTeX sichern
        if [ -f $2.aux ]; then
            cat *.aux | sort -u >$BACKUPDIR/$2.aux
            cat *.aux | grep citation | sort -u >$BACKUPDIR/$2.aux.bib
        fi
        echo $LATEX_MESSAGE
        LATEX_MESSAGE="AUX-Files unterschiedlich, starte LaTeX nochmals ..."
        FORCE_LATEX=0
        latex $2
        abort $? "LaTeX error ..."

        # wenn sich beim ersten mal Probleme mit BibTeX ergeben =>
        #   BibTeX aufrufen
        if [ $FIRST_LATEX -eq 1 ]; then
            FIRST_LATEX=0
            if diffit-bibtex $2 ; then bibtex-macro $2 ; fi
        fi

    done
    # letzten Zustand sichern
    if [ -f $2.aux ]; then
        cat *.aux | sort -u >$BACKUPDIR/$2.aux
        cat *.aux | grep citation | sort -u >$BACKUPDIR/$2.aux.bib
    fi
}



# Argument-Zahl pruefen
if [ $# -lt 4 ]; then
    echo usage: `basename $0` "<complete> <basename> <backupdir> <bibtex> [<bibtex-sources>]"
    echo '   complete  = 0/1 (fast/complete run)'
    echo '   basename  = name of texfile (without .tex suffix)'
    echo '   backupdir = name of backupdir (without leading slash)'
    echo '   bibtex    = 0/1 (disable/enable bibtex)'
    exit 1
fi
COMPLETE_RUN=$1
TEX_BASENAME=$2
BACKUPDIR=$3
RUN_BIBTEX=$4
if [ $RUN_BIBTEX -eq 1 ]; then
    if [ $# -ne 5 ]; then
        echo usage: `basename $0` "<complete> <basename> <backupdir> <bibtex> [<bibtex-sources>]"
        echo '   complete  = 0/1 (fast/complete run)'
        echo '   basename  = name of texfile (without .tex suffix)'
        echo '   backupdir = name of backupdir (without leading slash)'
        echo '   bibtex    = 0/1 (disable/enable bibtex)'
        exit 1
    fi 
    BIBTEX_SRC=$5
fi

# falls BACKUPDIR noch nicht existiert: erzeugen
if [ ! -d $BACKUPDIR ]; then
    mkdir $BACKUPDIR
fi

# Testet auf vorhandenes .aux-File und kompletten Lauf
if [ $COMPLETE_RUN -eq 1 -a ! -f $TEX_BASENAME.aux ]; then
    # .aux-File existiert nicht
    #  ==>  kompletter Durchlauf
    #       (fehlt: bei Fehlern anhalten)
    echo "Kompletter Lauf"
    bibtex-macro $TEX_BASENAME 
    latex-macro $? $TEX_BASENAME 
else
    # .aux-File existiert ODER kein kompletter Lauf

    FORCE=1
    if diffit-bibtex $TEX_BASENAME ; then
        # Test hat noetigen Lauf von BibTeX ergeben
        bibtex-macro $TEX_BASENAME 
        FORCE=$?
    fi
    latex-macro $FORCE $TEX_BASENAME 
fi
