#
# @command IMAGE
# @title   Einbinden von Grafiken
# @index
# @syntax  \#IMAGE\{optionen\}
# @desc Der Befehl {\tt \#IMAGE} bietet die M"oglichkeit, Bilder und
#       Grafiken in Dokumente zu integrieren. Das Bilddateiformat wird
#       an der Dateierweiterung erkannt und, falls notwendig, passend
#       f"ur das Output-Format konvertiert. Es werden die folgenden
#       Bilddateiformate unterst"utzt:
#       \begin{itemize}
#         \item GIF, Animated GIF ({\tt .gif})
#         \item JPEG ({\tt .jpg})
#         \item PNG ({\tt .png})
#         \item Tiff ({\tt .tif}, {\tt .tiff})
#         \item Postscript ({\tt .eps}, {\tt .ps})
#         \item XFig ({\tt .fig})
#         \item FigTeX ({\tt .fig})
#         \item Gnuplot ({\tt .plt})
#       \end{itemize}
# @example_begin
#  #IMAGE{
#      file=Plot/sin.plt,            % gnuplot-Datei einbinden
#      trans,                        % Transparenz (nur HTML-Version)
#      width=12cm,                   % LaTeX-Breite
#      scale=1.4,                    % Skalierungsfaktor (nur HTML-Version)
#      sign="Die \$sin\$-Funktion"   % Bildunterschrift
#  }
# @example_end
#
#   @optname file
#   @optargs <bilddatei> 
#      Eine notwendige Option. {\tt bilddatei} mu"s den gesamten (relativen oder absoluten) Pfad enthalten. 
#
#   @optname alt
#   @optargs <alternativbilddatei> 
#      Mit diesem Schalter kann das alternative Bild in die \LaTeXe-Version eingef"ugt werden.
#      Wird bei der alternativen Darstellung einigen Bildformaten (JPEG,PNG) in der
#      \LaTeXe-Version benutzt. 
#
#   @optname angle
#   @optargs <winkel> 
#      Mit dieser Option wird der Drehwinkel in der \LaTeXe-Version bestimmt (in Graden
#      gegen den Uhrzeigersinn). Dieser Parameter wird f"ur die FigTeX-Bilder ignoriert. 
#
#   @optname (no)center 
#      Bild (nicht) zentrieren. 
#
#   @optname factor
#   @optargs <skalierungsfaktor> 
#      Skalierungsfaktor des FigTeX-Bildes in \LaTeXe-Version. Erlaubt sind die reellen
#      Werte gr"o"ser 0. Bei diesem Faktor werden die Optionen width und height ignoriert.
#      \\Beispiel: {\tt faktor=0.5}
#      \\Defaultwert: {\tt 1}
#
#   @optname figtex 
#      Es handelt sich um ein FigTeX-Bild. 
#
#   @optname (no)figure 
#      Bild (nicht) als Gleitobjekt setzen. 
#
#   @optname gnuplot 
#      Es handelt sich um eine Gnuplot-Grafik (falls Dateierweiterung nicht .plt). 
#
#   @optname hangle
#   @optargs <winkel> 
#      Mit dieser Option wird der Drehwinkel in der HTML-Version drehwinkel bestimmt
#      (in Graden gegen den Uhrzeigersinn). Erlaubte Werte sind 90, 180 und 270.
#      Diese Option hat keinen Einflu"s auf die GIF-Bilder. 
#
#   @optname height
#   @optargs <h"ohe> 
#      H"ohe des Bildes in \LaTeXe-Version. 
#      \\Beispiel: {\tt height=12cm}
#      \\Defaultwert: Originalh"ohe. 
#
#   @optname hheight
#   @optargs <html-h"ohe> 
#      H"ohe des Bildes in HTML-Version (in Pixel). 
#      \\Beispiel: {\tt height=300}
#      \\Defaultwert: Originalh"ohe. 
#
#   @optname htmlonly 
#      Bild wird nur in HTML-Version sichtbar. 
#
#   @optname hwidth
#   @optargs <html-breite> 
#      Breite des Bildes in HTML-Version (in Pixel). 
#      \\Beispiel: {\tt hwidth=230}
#      \\Defaultwert: Originalbreite. 
#
#   @optname image
#   @optargs <bildnummer> 
#      Bild mit der Nummer bildnummer aus einem Animated-GIF-Bild in
#      \LaTeXe-Version einf"ugen. 
#      \\Defaultwert: {\tt 1}
#
#   @optname latexonly 
#      Bild wird nur in \LaTeXe-Version sichtbar. 
#
#   @optname label
#   @optargs <labelbezeichner> 
#      Bild wird im Dokument mit labelbezeichner markiert. Mit dem Master-Befehl
#      #IMAGEREF{<labelbezeichner>} kann man auf das Label eine Referenz anlegen. 
#
#   @optname nospace
#      Vor dem Bild wird kein Extra-Zwischenraum eingef"ugt.
#
#   @optname scale
#   @optargs <qualit"atsfaktor> 
#      Qualit"atsfaktor f"ur die generierten GIF-Bilder. Es sind die reellen
#      Werte zwischen 1 und 4 erlaubt. 
#      \\Defaultwert: {\tt 1}
#
#   @optname sign
#   @optargs \dq bildunterschrift\dq
#      Bildunterschrift angeben. Alle \LaTeXe-Befehle sind erlaubt. Die
#      Sonderzeichen ($\backslash$, \dq , , , \$, =, \{, \}) m"ussen mit $\backslash$
#      (backslash) gesch"utzt werden. 
#
#   @optname (no)trans 
#      Bild wird in HTML-Version (nicht) transparent. 
#
#   @optname width
#   @optargs <breite> 
#      Breite des Bildes in \LaTeXe-Version. 
#      \\Beispiel: {\tt width=9cm}
#      \\Defaultwert: Originalbreite. 
# @end

# Wird auch von anderen Modulen gebraucht 
sub image_work {
  package image;

  my ($wert,$flag);
  local ($use_name,$use_sign,$use_latexonly);
  local $use_alt_name;
  local $use_trans = $main::USE_TRANSPARENT;
  local $use_figure = $main::USE_FIGURE;
  local $figure_options = $main::FIGURE_OPTIONS;
  local $use_center = $main::USE_CENTER;
  local $use_scale = $main::USE_SCALE;
  local $use_factor = $main::USE_FACTOR;
  local $use_antialias = $main::USE_IMAGES_ANTIALIAS;  
  local ($use_width,$use_height);
  local ($use_hwidth,$use_hheight);
  local $use_htmlonly;
  local ($use_relativ_name);
  local ($use_label);
  local ($use_image);
  local ($use_nospace);
  local ($use_angle,$use_hangle);
  local ($use_figtex,$use_gnuplot);
  local ($use_local_name,$use_type);
  local ($noticy_key);
  local ($mtime);
  local $status;          # gleich null, wenn die
                          # Konvertierung fehlerfrei
  my (@stat_liste);
  my $file_name;          # Dateiname, der angegeben ist
  my $file_alt_name;      # alternativer Dateiname, der angegeben ist

  my $report_state = $main::REPORT_WRITE; # merkt altes Report-Zustand
                          # wird in die NTC-Datei eingetragen
  local $COMMAND = shift @_;
  local (@PARAMS) = @_;   # uebergebene Liste

  local ($IMAGE2);        # Steuerungsflag, kommt von IMAGE2
  local ($NO_BOT_SPACE);  # Wenn 1, dann bekommt man kein Abstand zwischen dem Bild
                          # und nachvolgendem Text.
  
  # wenn keine Erlaubung zum Schreiben, dann Ausstieg
  unless ($main::TEXFILE_WRITE_OK) {return;}

  # Flags setzen
  foreach $term (@PARAMS) {  
    $term =~ s/\"/\\\"/g if ($COMMAND ne "IMAGE");  
    ($flag,$wert)=&main::my_quotewords("=",0,$term);
    $flag =~ s/(\s*)//g;      # Trennzeichen loeschen
    SWITCH: {
      if ($flag eq "file") {$use_name=$wert; $file_name=$wert; last SWITCH;}
      if ($flag eq "alt") { $use_alt_name=$wert; $file_alt_name=$wert; last SWITCH; }
      if ($flag eq "sign") {$use_sign=$wert; last SWITCH;}
      if ($flag eq "trans") {$use_trans="1"; last SWITCH;}
      if ($flag eq "notrans") {$use_trans=""; last SWITCH;}
      if ($flag eq "figure") {$use_figure="1"; last SWITCH;}
      if ($flag eq "nofigure") {$use_figure=""; last SWITCH;}
      if ($flag eq "center") {$use_center="1"; last SWITCH;}
      if ($flag eq "nocenter") {$use_center=""; last SWITCH;}
      if ($flag eq "figtex") {$use_figtex="1"; last SWITCH;}
      if ($flag eq "gnuplot") {$use_gnuplot="1"; last SWITCH;}
      if ($flag eq "latexonly") {$use_latexonly="1"; last SWITCH;}
      if ($flag eq "htmlonly") {$use_htmlonly="1"; last SWITCH;}
      if ($flag eq "width") {$use_width=$wert; last SWITCH;}
      if ($flag eq "height") {$use_height=$wert; last SWITCH;}
      if ($flag eq "factor") {$use_factor=$wert; last SWITCH;}
      if ($flag eq "hwidth") {$use_hwidth=$wert; last SWITCH;}
      if ($flag eq "hheight") {$use_hheight=$wert; last SWITCH;}
      if ($flag eq "angle") {$use_angle=$wert; last SWITCH;}
      if ($flag eq "hangle") {$use_hangle=$wert; last SWITCH;}
      if ($flag eq "image") {$use_image=$wert; last SWITCH;}
      if ($flag eq "scale") {$use_scale=$wert; last SWITCH;}
      if ($flag eq "label") {$use_label=$wert; $use_figure="1"; last SWITCH;}
      if ($flag eq "nospace") {$use_nospace="1"; last SWITCH;}
      
      # diese Schalter werden nur innerhalb der Paketen benutzt
      if ($flag eq "IMAGE2") {$IMAGE2=$wert; last SWITCH;}
      if ($flag eq "NO_BOT_SPACE") {$NO_BOT_SPACE=1; last SWITCH;}
      &main::my_exit("Unknown flag in IMAGE : $flag\n");
    }
  }

  # alternatives Bild
  if ($main::OUTPUTFORMAT eq "latex") {
    if ($use_alt_name) { $use_name = $use_alt_name; }
    if ($file_alt_name) { $file_name = $file_alt_name; }
  }

  &main::my_print("$COMMAND (l.$main::INPUTFILE_LINE) : File=$use_name\n");

  &to_flag_check;    # Syntax pruefen und wenn noetig die
                     # default-Werte setzen

  $use_relativ_name = &main::get_relativ_name($use_name);  # relativer Pfadname
  $use_local_name = &main::get_base_name($use_name);    # Basisname
  $use_type = &main::get_file_type($use_name);      # Dateierweiterung
  # Absoluten Pfad erkuendigen
  $use_dir=&main::get_abs_pfad($use_name);
  $use_name = "$use_dir/$use_local_name.$use_type";

  if ($use_type ne "fig") {$use_figtex = "";}
  else { $use_figtex = 1 if ($use_factor); }
  if ($use_type ne "gif") {$use_image = "";}
  if ($use_type eq "plt") {$use_gnuplot = 1;}
  if (($use_type eq "eps")&&(-B "$use_name")) {
    &main::my_msg("EPS-File \'$use_name\' is not ASCII.\n");
    return;
  }

  # fuer Statistik
  if ($use_figtex) {
    &main::statistic_count("figtex");
  } else {
    &main::statistic_count($use_type);
  }

  # den Notice-Schluessel bilden
  if ($use_figtex) {
   if ($main::OUTPUTFORMAT eq "html") {
    $noticy_key=&main::build_key($main::OUTPUTFORMAT,$file_name,$use_trans,
                   $use_scale,$use_hangle,$use_antialias);
   } else {
    $noticy_key=&main::build_key($main::OUTPUTFORMAT,$file_name,$use_width,$use_height,$use_factor);
   }
  } elsif ($use_image) {
   if ($main::OUTPUTFORMAT eq "html") {
    $noticy_key=&main::build_key($main::OUTPUTFORMAT,$file_name);
   } else {
    $noticy_key=&main::build_key($main::OUTPUTFORMAT,$file_name,$use_image);
   }
  } elsif (($use_type eq "ps")||($use_type eq "eps")||($use_type eq "tif")||($use_type eq "tiff")) {
   if ($main::OUTPUTFORMAT eq "html") {
    $noticy_key=&main::build_key($main::OUTPUTFORMAT,$file_name,$use_trans,$use_scale,$use_hangle);
   } else {
    $noticy_key=&main::build_key($main::OUTPUTFORMAT,$file_name);
   }
  } else {
   if ($main::OUTPUTFORMAT eq "html") {
    $noticy_key=&main::build_key($main::OUTPUTFORMAT,$file_name,$use_trans,$use_scale,
                   $use_hangle,$use_antialias);
   } else {
    $noticy_key=&main::build_key($main::OUTPUTFORMAT,$file_name);
   }
  }
  
  # Dateieigenschaften erkundigen (Dateigroesse und die letzte
  # Modifikationszeit
  @stat_liste = stat $use_name;
  $mtime = $stat_liste[9];

  # Schreiben in die Report-Version erlauben
  $main::REPORT_WRITE = 1;
  # Entsprechende Mechanismen aufrufen
  if (($main::OUTPUTFORMAT eq "html")&&(!($use_latexonly))) {&html_work;}
  if (($main::OUTPUTFORMAT eq "latex")&&(!($use_htmlonly))) {&latex_work;}
  if (($main::OUTPUTFORMAT eq "pdf")) {&pdf_work;}
  # Schreiben in die Report-Version zuruecksetzen
  $main::REPORT_WRITE = $report_state;
  
  # wenn alles geklappt, dann Notice-Eintrag machen
  if (!($status)) { #wenn erfolgreich dann eintragen
    $main::NOTICE_NEW{$noticy_key}="$mtime";
  } 
  return;

  # die Flags ueberpruefen
  sub to_flag_check {
    $use_name =~ s/(\s+)//g;
    $use_angle =~ s/(\s+)//g;
    unless ($use_name) {&main::my_exit("File is not statement.\n");}
    unless (-f $use_name) {&main::my_msg("File '$use_name' not found.\n");}
    if ($use_image =~ /(\D+)/) {
      &main::my_msg("Flag \'image\' desire number only.\n");
      $use_image = "";
    }
    if ($use_angle) {
      unless ($use_angle =~ /^(\d+\.?\d*|\.\d+)$/) {
        &main::my_msg("Not allowed value for \'angle$IMAGE2\' : $use_angle.\n");
        $use_angle = "";
      }
    }
    if (($use_hangle)&&(!($use_hangle =~ /(90|180|270)/))) {
      &main::my_msg("Flag \'hangle$IMAGE2\' must be one of: 90 180 270\n");
      $use_hangle = "";
    }
    if (($use_latexonly)&&($use_htmlonly)) {
      &main::my_msg("Flags latexonly and htmlonly in $COMMAND\n");
    }
    if (($use_scale)&&(($use_scale < 0.1)||($use_scale > 10))) {
      &main::my_msg("Not allowed value for \'scale$IMAGE2\' : $use_scale.\n");
      $use_scale=1.6;
    }
    # fuer LaTeX-Version ist die Scalierungsfaktor nicht relevant
    if ($main::OUTPUTFORMAT eq "latex") { $use_scale = ""; }
  
    # pruefe auf gueltigen Faktor
    if ($use_factor) {
      if ((!($use_factor =~ /^(\d+\.?\d*|\.\d+)$/))||($use_factor < 0)) {
        &main::my_msg("Not allowed value for \'factor$IMAGE2\' : $use_factor.\n");
        $use_factor = "";
      }
    }
  
    # Meldung fuer nicht moegliche Operationen fuer Bilder ausgeben
    if ($use_figtex) {
      # wenn die Groessen und der Faktor angegeben wurden
      if (($use_factor)&&(($use_width)||($use_height))) {
        &main::my_msg("Ignore image size, use of \'factor$IMAGE2\'.\n");
        $use_width="";
        $use_height="";
      }
      # wenn die beiden Groessen beim FigTeX-Bild eingestellt sind,
      # dann wird die Hoehe ignoriert
      if (($use_width)&&($use_height)) {
        &main::my_msg("Ignore \'height$IMAGE2\' value (figtex).\n");
      }
      &main::my_msg("Ignore \'angle\' value (figtex).\n") if ($use_angle);
    } else {
      if ($use_factor) { &main::my_msg("Ignore \'factor$IMAGE2=$use_factor\' for not figtex-image.\n"); }
    }
    $figure_options="htb" unless ($figure_options); 
  }
  
  # prueft, ob eine Konvertirungs-Routine notwendig ist
  sub ok_to_work {
    if ($main::MAKE_ALL) {return 1;}
    if ($main::NOT_NTCFILE) {return 0;}
    if (((exists $main::NOTICE_OLD{$noticy_key})
      and($main::NOTICE_OLD{$noticy_key} == $mtime))
      or (exists $main::NOTICE_NEW{$noticy_key})) {return 0;} 
    return 1;
  }
  
  # wandelt einen Dateinamen in einen LaTeX-Namen um (Unix-Style)
  sub filename_to_latex {
    my (@dirs) = File::Spec->splitdir($_[0]);
    return join("/", @dirs);
  }
  # wandelt einen Dateinamen in einen HTML-Namen um (Unix-Style)
  sub filename_to_html {
    my (@dirs) = File::Spec->splitdir($_[0]);
    return join("/", @dirs);
  }

  # Diese Subroutine konvertiert Master-Kode in HTML-Kode        #
  # Die Grafiken werden dabei automatisch ins GIF-Format umgewandelt  #
  sub html_work {
    my ($new_name);
    my $width;
    my $height;
    SWITCH: {
      if (($use_type eq "ps")||($use_type eq "eps")) {$new_name = &ps_to_html; last SWITCH;}
      if ($use_type eq "fig") {
        if ($use_figtex) {$new_name = &figtex_to_html; last SWITCH;}
        else {$new_name = &fig_to_html; last SWITCH;}
      }
      if (($use_type eq "tif")||($use_type eq "tiff")) {$new_name = &tiff_to_html; last SWITCH;}
      if ($use_type eq "gif") {$new_name = &gif_to_html; last SWITCH;}
      if ($use_type eq "jpg") {$new_name = &jpg_to_html; last SWITCH;}
      if ($use_type eq "png") {$new_name = &png_to_html; last SWITCH;}
      if ($use_type eq "pstex_t") {$new_name = &pstex_to_html; last SWITCH;}
      if ($use_gnuplot) { $new_name = &plot_to_html; last SWITCH; }
      &main::my_exit("Unkrown expansion '.$use_type'\n");
    }
    if ($use_hwidth) {$width="WIDTH=$use_hwidth";}
    if ($use_hheight) {$height="HEIGHT=$use_hheight";}
    &main::write_to_texfile(
      "% *** Master-Generator - $COMMAND fuer HTML ***\n".
      "% BEGIN : $use_local_name.$use_type\n") unless ($IMAGE2);
    &main::write_to_texfile(
      "\\begin{rawhtml}\n".
      "\t<P><P>\n".
      "\\end{rawhtml}\n") if ((!($IMAGE2))&&(!($use_nospace)));
    
    # Dieses Teil braucht man fuer die Zaehlung von
    # Abbildungen in der HTML-Version.
    # Nur dann kann man die Bilder in der HTML-Version referenzieren.
    &main::write_to_texfile(
      "\\begin{latexonly}\n".
      "\\begin{figure}[$figure_options]\n") if ($use_figure);
    &main::write_to_texfile("\t\\caption{\\label{$use_label}}\n") if ($use_label);
    &main::write_to_texfile(
      "\\end{figure}\n".
      "\\end{latexonly}\n") if ($use_figure);
    
    my ($html_name) = filename_to_html($new_name);
    &main::write_to_texfile("\\begin{rawhtml}\n");
    &main::write_to_texfile("\t<CENTER>\n") if ($use_center);
    &main::write_to_texfile("\t<A NAME=$use_label>\n") if ($use_label);
    &main::write_to_texfile("\t   <IMG SRC=\"$html_name\" $width $height>\n");
    &main::write_to_texfile("\t<\/A>\n") if ($use_label);
    &main::write_to_texfile("\t<BR>\n") unless ($use_nospace);
    &main::write_to_texfile("\t</CENTER>\n") if ($use_center);
    &main::write_to_texfile("\\end{rawhtml}\n");
    
    if ($use_sign) {
      &main::write_to_texfile(
        "\\begin{rawhtml}\n".
        "\t<$main::HTML_SIGN_FONT>\n".
        "\\end{rawhtml}\n"
      ) if ($main::HTML_SIGN_FONT);
      &main::write_to_texfile("\\begin{center}\n") if ($use_center);
      &main::write_to_texfile("Abbildung \\ref{$use_label}: ") if ($use_label);
      &main::write_to_texfile("$use_sign"."\n") if ($use_sign);
      &main::write_to_texfile("\\end{center}\n") if ($use_center);
      &main::write_to_texfile(
        "\\begin{rawhtml}\n".
        "\t<\/$main::HTML_SIGN_FONT>\n".
        "\\end{rawhtml}\n"
      ) if ($main::HTML_SIGN_FONT);
    }
  
    &main::write_to_texfile(
      "\\begin{rawhtml}\n".
      "\t<P>\n".
      "\\end{rawhtml}\n"
    ) unless ($NO_BOT_SPACE);
    &main::write_to_texfile("% END : $use_name fuer HTML\n") unless ($IMAGE2);
  }
  
  #
  # Konvertiert PS-Dateien ins GIF-Format
  #
  sub ps_to_html {
    my $DIR_TARGET = File::Spec->catdir("Images","Ps");
    my $NW_NAME    = File::Spec->catdir("Images","Ps","$use_local_name.gif");
  
    &main::get_underdir("$DIR_TARGET");
    
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$main::OUTPUT_DIR,$NW_NAME);
    if ((&ok_to_work)||(!(-f $FULL_NAME))) {
      $status=&main::ps_to_gif("$use_name","$DIR_TARGET","$use_trans","$use_scale","$use_hangle","");
    }
    return $NW_NAME;
  }
  
  #
  # Konvertiert Tiff-Dateien ins GIF-Format
  #
  sub tiff_to_html {
    my $DIR_TARGET = File::Spec->catdir("Images","Tiff");
    my $NW_NAME    = File::Spec->catdir("Images","Tiff","$use_local_name.gif");
  
    &main::get_underdir("$DIR_TARGET");
    
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$main::OUTPUT_DIR,$NW_NAME);
    if ((&ok_to_work)||(!(-f $FULL_NAME))) {
      $status=&main::tiff_to_gif("$use_name","$DIR_TARGET","$use_trans","$use_scale","$use_hangle","");
    }
    return $NW_NAME;
  }
  
  #
  # Konvertiert fig-Dateien ins Gif-Format
  #
  sub fig_to_html {
    my $DIR_TARGET = File::Spec->catdir("Images","Fig");
    my $NW_NAME    = File::Spec->catdir("Images","Fig","$use_local_name.gif");
  
    &main::get_underdir("$DIR_TARGET");
  
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$main::OUTPUT_DIR,$NW_NAME);
    if ((&ok_to_work)||(!(-f $FULL_NAME))) {
      $status=&main::fig_to_gif("$use_name","$DIR_TARGET","$use_trans","$use_scale",
                    "$use_hangle","$use_antialias");
    }
    return $NW_NAME;
  }
  
  #
  # Konvertiert figtex-Dateien ins Gif-Format
  #
  sub figtex_to_html {
    my $DIR_TARGET = File::Spec->catdir("Images","Figtex");
    my $NW_NAME    = File::Spec->catdir("Images","Figtex","$use_local_name.gif");
  
    &main::get_underdir("$DIR_TARGET");
  
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$main::OUTPUT_DIR,$NW_NAME);
    if ((&ok_to_work)||(!(-f $FULL_NAME))) {
      $status=&main::figtex_to_gif("$use_name","$DIR_TARGET","$use_trans","$use_scale",
                     "$use_hangle","$use_antialias");
    }
    return $NW_NAME;
  }
  
  #
  # Konvertiert GnuPlot-Dateien ins Gif-Format
  #
  sub plot_to_html {
    my $DIR_TARGET = File::Spec->catdir("Images","Plot");
    my $NW_NAME    = File::Spec->catdir("Images","Plot","$use_local_name.gif");
  
    &main::get_underdir("$DIR_TARGET");
  
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$main::OUTPUT_DIR,$NW_NAME);
    if ((&ok_to_work)||(!(-f $FULL_NAME))) {
      $status=&main::gnuplot_to_gif("$use_name","$DIR_TARGET","$use_trans","$use_scale",
                      "$use_hangle","$use_antialias");
    }
    return $NW_NAME;
  }
  
  #
  # Convertiert pstex_t-Dateien ins GIF-Format fuer HTML
  #
  sub pstex_to_html {
    my $DIR_TARGET = File::Spec->catdir("Images","Figtex");
    my $NW_NAME    = File::Spec->catdir("Images","Figtex","$use_local_name.gif");
    
    &main::get_underdir("$DIR_TARGET");
  
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$main::OUTPUT_DIR,$NW_NAME);
    if ((&ok_to_work)||(!(-f $FULL_NAME))) {
      $status=&main::pstex_to_gif("$use_name","$DIR_TARGET","$use_trans","$use_scale",
                    "$use_hangle","$use_antialias");
    }
    return $NW_NAME;
  }
  
  #
  # Kopiert die Gif-Datei in ensprechendes Verzeichnis (wenn '-copy' - Flag)
  # 
  sub gif_to_html {
    my $DIR_TARGET = File::Spec->catdir("Images","Gif");
    my $NW_NAME    = File::Spec->catdir("Images","Gif","$use_local_name.$use_type");
  
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$main::OUTPUT_DIR,$NW_NAME);
    if (($main::COPY_SOURCE)&&((&ok_to_work)||(!(-f $FULL_NAME)))) {
      &main::my_print("&main::copy $use_name to $FULL_NAME\n");
      &main::get_underdir("$DIR_TARGET");
      &main::copy("$use_name", File::Spec->catdir($main::WORK_DIR,$main::OUTPUT_DIR,$DIR_TARGET));
      $main::SOURCE_IS_CHANGE=1;
    }
  
    if ($main::COPY_SOURCE) { return $NW_NAME;}
    else {
      if ($main::RELATIVE_PFAD) {return File::Spec->catdir("..",$use_relativ_name)}
      else { return $use_name; }
    }
  }
  
  #
  # Kopiert die Jpeg-Datei in ensprechendes Verzeichnis (wenn '-copy' - Flag)
  # 
  sub jpg_to_html {
    my $DIR_TARGET = File::Spec->catdir("Images","Jpeg");
    my $NW_NAME    = File::Spec->catdir("Images","Jpeg","$use_local_name.$use_type");
  
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$main::OUTPUT_DIR,$NW_NAME);
    if (($main::COPY_SOURCE)&&((&ok_to_work)||(!(-f $FULL_NAME)))) {
      &main::my_print("&main::copy $use_name to $FULL_NAME\n");
      &main::get_underdir("$DIR_TARGET");
      &main::copy("$use_name",File::Spec->catdir($main::WORK_DIR,$main::OUTPUT_DIR,$DIR_TARGET));
      $main::SOURCE_IS_CHANGE=1;
    }
  
    if ($main::COPY_SOURCE) { return $NW_NAME;}
    else {
      if ($main::RELATIVE_PFAD) {return File::Spec->catdir("..",$use_relativ_name);}
      else { return $use_name; }
    }
  }
  
  #
  # Kopiert die Png-Datei in ensprechendes Verzeichnis (wenn '-copy' - Flag)
  # 
  sub png_to_html {
    my $DIR_TARGET = File::Spec->catdir("Images","Png");
    my $NW_NAME    = File::Spec->catdir("Images","Png","$use_local_name.$use_type");
  
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$main::OUTPUT_DIR,$NW_NAME);
    if (($main::COPY_SOURCE)&&((&ok_to_work)||(!(-f $FULL_NAME)))) {
      &main::my_print("&main::copy $use_name to $FULL_NAME\n");
      &main::get_underdir("$DIR_TARGET");
      &main::copy("$use_name", File::Spec->catdir($main::WORK_DIR,$main::OUTPUT_DIR,$DIR_TARGET));
      $main::SOURCE_IS_CHANGE=1;
    }
  
    if ($main::COPY_SOURCE) { return $NW_NAME;}
    else {
      if ($main::RELATIVE_PFAD) {return File::Spec->catdir("..",$use_relativ_name);}
      else { return $use_name; }
    }
  }
  
  # Diese Subroutine konvertiert Master-Kode in Latex-Kode        #
  # Die Grafiken werden dabei automatisch ins PS-Format umgewandelt    #
  sub latex_work {
    my ($new_name);
    my ($label);
    my ($width,$height,$angle);
    SWITCH: {
      if (($use_type eq "ps")||($use_type eq "eps")) {$new_name = &ps_to_latex; last SWITCH;}
      if ($use_type eq "fig") {
        if ($use_figtex) {$new_name = &figtex_to_latex; last SWITCH;}
        else {$new_name = &fig_to_latex; last SWITCH;}
      }
      if (($use_type eq "tif")||($use_type eq "tiff")) {$new_name = &tiff_to_latex; last SWITCH;}
      if ($use_type eq "gif") {$new_name = &gif_to_latex; last SWITCH;}
      if ($use_type eq "pstex_t") {$new_name = &pstex_to_latex; last SWITCH;}
      if ($use_gnuplot) { $new_name = &plot_to_latex; last SWITCH; }
      if ($use_type eq "jpg") {&main::my_msg("JPEG not visible for LaTeX\n"); return;}
      if ($use_type eq "png") {&main::my_msg("PNG not visible for LaTeX\n"); return;}
      &main::my_exit("Unkrown expansion '.$use_type'\n");
    }
    my ($latex_name) = filename_to_latex($new_name);
    if ($use_label) {
      $label="\\label{$use_label}"; 
    }
    &main::write_to_texfile(
      "% *** Master-Generator - $COMMAND fuer LaTeX ***\n".
      "% BEGIN : $use_local_name.$use_type\n") unless ($IMAGE2);
    &main::write_to_texfile("\\begin{figure}[$figure_options]\n") if ($use_figure);
    &main::write_to_texfile("\t\\begin{center}\n") if ($use_center);
    if (($use_type eq "pstex_t")||(($use_type eq "fig")&&($use_figtex))) {
      &main::write_to_texfile("\t\t\\input{$latex_name}\n");
    } else {
      $width=",width=$use_width" if ($use_width);
      $height=",height=$use_height" if ($use_height);
      $angle=",angle=$use_angle" if ($use_angle);
      &main::write_to_texfile("\t\\epsfig{file=$latex_name$width$height$angle}\n");
    }
    if ($use_figure) {
      if ($use_sign) {
        # bei der Caption-Umgebung einige Befehle
        # mit \protect-Befehl schuetzen
        $use_sign =~ s/\\(begin|end|\\)/\\protect\\$1/g;
    
        &main::write_to_texfile("\t\\caption{$label \\small ".$use_sign."}\n");
      } else {
        &main::write_to_texfile("\t\\caption{$label}\n");
      }
    } else {
      &main::write_to_texfile("\n\t{\\small \\rm ".$use_sign."}\n") if ($use_sign);
    }
    &main::write_to_texfile("\t\\end{center}\n") if ($use_center);
    &main::write_to_texfile("\\end{figure}\n") if ($use_figure);
    &main::write_to_texfile("% END : $use_name fuer LaTeX\n") unless ($IMAGE2);
  }

  # Diese Subroutine konvertiert Master-Kode in PDF-Kode
  # Die Grafiken werden dabei automatisch ins PS-Format umgewandelt
  sub pdf_work {
    my ($new_name);
    my ($label);
    my ($width,$height,$angle);
    SWITCH: {
      if (($use_type eq "ps")||($use_type eq "eps")) {$new_name = &ps_to_latex; last SWITCH;}
      if ($use_type eq "fig") {
        if ($use_figtex) {$new_name = &figtex_to_latex; last SWITCH;}
        else {$new_name = &fig_to_latex; last SWITCH;}
      }
      if (($use_type eq "tif")||($use_type eq "tiff")) {$new_name = &tiff_to_latex; last SWITCH;}
      if ($use_type eq "gif") {$new_name = &gif_to_pdf; last SWITCH;}
      if ($use_type eq "pstex_t") {$new_name = &pstex_to_latex; last SWITCH;}
      if ($use_gnuplot) { $new_name = &plot_to_latex; last SWITCH; }
      if ($use_type eq "jpg") {&main::my_msg("JPEG not visible for PDF\n"); return;}
      if ($use_type eq "png") {&main::my_msg("PNG not visible for PDF\n"); return;}
      &main::my_exit("Unkrown expansion '.$use_type'\n");
    }
    my ($latex_name) = filename_to_latex($new_name);
    if ($use_label) {
      $label="\\label{$use_label}"; 
    }
    &main::write_to_texfile(
      "% *** Master-Generator - $COMMAND fuer PDF ***\n".
      "% BEGIN : $use_local_name.$use_type\n") unless ($IMAGE2);
    &main::write_to_texfile("\\begin{figure}[$figure_options]\n") if ($use_figure);
    &main::write_to_texfile("\t\\begin{center}\n") if ($use_center);
    if (($use_type eq "pstex_t")||(($use_type eq "fig")&&($use_figtex))) {
      &main::write_to_texfile("\t\t\\input{$latex_name}\n");
    } else {
      my $tmp_name;
      $width=",width=$use_width" if ($use_width);
      $height=",height=$use_height" if ($use_height);
      if ($use_type eq "gif") {
        # Dimensionen holen, dabei korrekten Dateinamen benutzen
        if (File::Spec->file_name_is_absolute($new_name)) {
          $tmp_name = $new_name;
        } else {
          $tmp_name = File::Spec->catfile($main::WORK_DIR,$new_name);
        }
        ($w, $h, $id) = &Image::Size::imgsize($tmp_name);
        &main::my_exit("Can't get size for $tmp_name\n  Reason: $id ($main::WORK_DIR)\n") if ($w=="" || $h=="");
        &main::write_to_texfile("\t\t\\includegraphics[bb=0 0 $w $h$width$height]{$latex_name}\n");
      } else {
        $angle=",angle=$use_angle" if ($use_angle);
        &main::write_to_texfile("\t\\epsfig{file=$latex_name$width$height$angle}\n");
      }
    }
    if ($use_figure) {
      if ($use_sign) {
        # bei der Caption-Umgebung einige Befehle
        # mit \protect-Befehl schuetzen
        $use_sign =~ s/\\(begin|end|\\)/\\protect\\$1/g;
    
        &main::write_to_texfile("\t\\caption{$label \\small ".$use_sign."}\n");
      } else {
        &main::write_to_texfile("\t\\caption{$label}\n");
      }
    } else {
      &main::write_to_texfile("\n\t{\\small \\rm ".$use_sign."}\n") if ($use_sign);
    }
    &main::write_to_texfile("\t\\end{center}\n") if ($use_center);
    &main::write_to_texfile("\\end{figure}\n") if ($use_figure);
    &main::write_to_texfile("% END : $use_name fuer PDF\n") unless ($IMAGE2);
  }

  #
  # Konvertiert gif-Dateien in PNG
  #
  sub gif_to_pdf {
    my $NW_NAME   = File::Spec->catfile($main::OUTPUT_DIR,$use_local_name."_gif.png");
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$NW_NAME);
    if ((&ok_to_work)||(!(-f $FULL_NAME))) {
      $status=&main::gif_to_png("$use_name","$main::OUTPUT_DIR","$use_image");
    }
    return "$NW_NAME";
  }
  
  
  
  #
  # Postscript fuer LaTeX
  #
  sub ps_to_latex {
    if (&ok_to_work) { $main::SOURCE_IS_CHANGE = 1; }
    return $use_name;
  }
  
  #
  # Pstex_t fuer LaTeX
  #
  sub pstex_to_latex {
    if (&ok_to_work) { $main::SOURCE_IS_CHANGE = 1; }
    return $use_name;
  }
  
  #
  # Konvertiert fig-Dateien in Postscript
  #
  sub fig_to_latex {
    my $NW_NAME   = File::Spec->catfile($main::OUTPUT_DIR,$use_local_name."_fig.ps");
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$NW_NAME);
  
    if ((&ok_to_work)||(!(-f $FULL_NAME))) {
      $status=&main::fig_to_ps("$use_name","$main::OUTPUT_DIR","$use_width","$use_height");
    }
    return $NW_NAME;
  }
  
  #
  # Bindet figtex-Dateien in LaTeX ein
  #
  sub figtex_to_latex {
    my $NW_NAME    = File::Spec->catfile($main::OUTPUT_DIR,"$use_local_name.pstex_t");
    my $PS_NAME    = File::Spec->catfile($main::OUTPUT_DIR,"$use_local_name.pstex");
    my $FULL_NAME1 = File::Spec->catfile($main::WORK_DIR,$NW_NAME);
    my $FULL_NAME2 = File::Spec->catfile($main::WORK_DIR,$PS_NAME);
  
    if ((&ok_to_work)||(!(-f $FULL_NAME1))||(!(-f $FULL_NAME2))) {
      $status=&main::figtex_to_pstex("$use_name","$main::OUTPUT_DIR","$use_width","$use_height","$use_factor");
    }
    return $NW_NAME;
  }
  
  #
  # Bindet GnuPlot-Dateien in LaTeX ein
  #
  sub plot_to_latex {
    my $NW_NAME   = File::Spec->catfile($main::OUTPUT_DIR,$use_local_name."_plot.ps");
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$NW_NAME);
  
    if ((&ok_to_work)||(!(-f $FULL_NAME))) {
      $status=&main::gnuplot_to_ps("$use_name","$main::OUTPUT_DIR");
    }
    return $NW_NAME;
  }
  
  #
  # Konvertiert gif-Dateien in Postscript
  #
  sub gif_to_latex {
    my $NW_NAME   = File::Spec->catfile($main::OUTPUT_DIR,$use_local_name."_gif.ps");
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$NW_NAME);
  
    if ((&ok_to_work)||(!(-f $FULL_NAME))) {
      $status=&main::gif_to_ps("$use_name","$main::OUTPUT_DIR","$use_image");
    }
    return "$NW_NAME";
  }
  
  #
  # Konvertiert Tiff-Dateien in Postscript
  #
  sub tiff_to_latex {
    my $NW_NAME   = File::Spec->catfile($main::OUTPUT_DIR,$use_local_name."_tiff.ps");
    my $FULL_NAME = File::Spec->catfile($main::WORK_DIR,$NW_NAME);
  
    if ((&ok_to_work)||(!(-f $FULL_NAME))) {
      $status=&main::tiff_to_ps("$use_name","$main::OUTPUT_DIR");
    }
    return "$NW_NAME";
  }
  
}

$MANAGEFUNKTION->{IMAGE} = \&image_work;

1;
