#
# @command INTRO
# @title   Standard-Dokumenttitel erzeugen
# @index   
# @syntax  \#INTRO\{optionen\}
# @desc    Erzeugt einen Standard-Dokumenttitel, der den Autor,
#          den Titel und weitere Information zum Dokument enthalten
#          kann.
# @example_begin
#   #INTRO{
#      title  ="Problemf\"alle",
#      author ="Oliver Vornberger",
#      dept   ="Fachbereich Mathematik/Informatik",
#      uni    ="Universit\"at Osnabr\"uck",
#      city   ="D-49069 Osnabr\"uck",
#      email  ="oliver@uos.de",
#      online ="http://www-lehre.inf.uos.de/mas2tex/problem",
#      pdf    ="../problem.pdf"
#   }
# @example_end
#
#   @optname title
#   @optargs \dq <Titel>\dq 
#       Titel des Dokumentes. Dieser wird in fetter und gr"osserer
#       Schrift dargestellt.
#
#   @optname author
#   @optargs \dq <Autor>\dq 
#       Name des Autors.
#
#   @optname url
#   @optargs \dq <URL>\dq 
#       URL zur Homepage des Autors
#
#   @optname dept
#   @optargs \dq <Abteilung>\dq 
#       Name der Abteilung des Autors
#
#   @optname uni
#   @optargs \dq <Firma>\dq 
#       Name der Firma oder Institution des Autors
#
#   @optname city
#   @optargs \dq <Name>\dq 
#       Ortsname des Authors oder der zugeh"origen Firma
#       bzw. Institution.
#
#   @optname email
#   @optargs \dq <E-Mail>\dq 
#       E-Mail Adresse des Autors.
#
#   @optname online
#   @optargs \dq <URL>\dq
#       URL zur Online-Version des Dokumentes (HTML-Format)
#
#   @optname pdf
#   @optargs \dq <URL>\dq
#       URL zur Online-Version des Dokumentes (PDF-Format)
#
#   @optname dsign
#   @optargs \dq <Text>\dq
#       Text, der im Link zur PDF-Version auftritt.
#
# @end

# Mas2TeX-Befehle definieren
$MANAGEFUNKTION->{INTRO}     = \&intro_work;

# INTRO
sub intro_work {
  my ($wert,$flag);
  my $use_title  = "";
  my $use_author = "";
  my $use_url    = "";
  my $use_dept   = "";
  my $use_uni    = "";
  my $use_city   = "";
  my $use_email  = "";
  my $use_online = "";
  my $use_pdf    = "";
  my $use_dsign  = "PDF-Version";
  my $COMMAND  = shift @_;
  my (@PARAMS) = @_;

  # Flags setzen
  foreach $term (@PARAMS) {    
    ($flag,$wert)=&main::my_quotewords("=",0,$term);
    $flag =~ s/(\s*)//g;      # Trennzeichen loeschen
    SWITCH: {
      if ($flag eq "title") {$use_title=$wert; last SWITCH;}
      if ($flag eq "author") {$use_author=$wert; last SWITCH;}
      if ($flag eq "url") {$use_url=$wert; last SWITCH;}
      if ($flag eq "dept") {$use_dept=$wert; last SWITCH;}
      if ($flag eq "uni") {$use_uni=$wert; last SWITCH;}
      if ($flag eq "city") {$use_city=$wert; last SWITCH;}
      if ($flag eq "email") {$use_email=$wert; last SWITCH;}
      if ($flag eq "online") {$use_online=$wert; last SWITCH;}
      if ($flag eq "pdf") {$use_pdf=$wert; last SWITCH;}
      if ($flag eq "dsign") {$use_dsign=$wert; last SWITCH;}
      &main::my_exit("Unknown flag in $COMMAND : $flag\n");
    }
  }

  #if ($use_online) {
  #  $use_online="online: "."$use_online";
  #}
  #if ($use_email) {
  #  $use_email="email: "."$use_email";
  #}

  &main::my_print("$COMMAND : $main::MASTERFILE.$main::EXT\n");
  &main::master_write("% *** Master-Generator: Intro of Document $main::TEXFILE ***\n");
  if ($main::OUTPUTFORMAT eq "html") {
    if ($use_url)  {
      $use_author="<A HREF=\"$use_url\">$use_author<\/A>";
    }  
    &main::master_write(
      "\\begin{rawhtml}\n".
      "\t<P><H1 ALIGN=\"CENTER\">\n".
      "\\end{rawhtml}\n".
      "$use_title\n".
      "\\begin{rawhtml}\n".
      "<\/H1>\n".
      "\t<P ALIGN=\"CENTER\"><STRONG>\n".
      "\\end{rawhtml}\n".
      "$use_author\n".
      "\\begin{rawhtml}\n".
      "\t<BR>\n".
      "\\end{rawhtml}\n".
      "$use_dept\n".
      "\\begin{rawhtml}\n".
      "\t<BR>\n".
      "\\end{rawhtml}\n".
      "\t$use_uni\n".
      "\\begin{rawhtml}\n".
      "\t<BR>\n".
      "\\end{rawhtml}\n".
      "\t$use_city\n".
      "\\begin{rawhtml}\n".
      "\t<BR>\n".
      "\t<A HREF=\"mailto:$use_email\">$use_email<\/A><\/P><\/STRONG>\n".
      "\t<P ALIGN=\"CENTER\"><STRONG><\/STRONG><\/P>\n".
      "\t<P ALIGN=\"LEFT\"><\P>\n".
      "\t<P><DIV ALIGN=\"CENTER\">\n".
      "\t\t<A HREF=\"$use_online\"><TT>$use_online<\/TT><\/A>\n".
      "\t<\/DIV>\n".
      "\\end{rawhtml}\n"
    );
    if ($use_pdf) {
      &main::master_write(
        "\\begin{rawhtml}\n".
        "\t<P><DIV ALIGN=\"CENTER\">\n".
        "\t<A HREF=\"$use_pdf\">\n".
        "\\end{rawhtml}\n".
        "\t$use_dsign\n".
        "\\begin{rawhtml}\n".
        "\t<\/A>\n".
        "\t<\/DIV>\n".
        "\\end{rawhtml}\n"
      );
    }
    #&main::master_write("\\end{rawhtml}\n");
  } elsif ($main::OUTPUTFORMAT eq "latex") {
    &main::master_write(
      "\t\\vspace\{3cm\}\n".
      "\t\\begin\{center\}\n".
      "\t\t\{\\huge $use_title\}\n\n".
      "\t\t\\vspace\{1.5cm\}\n".
      "\t\t\{\\Large $use_author\}\n\n".
      "\t\t\\vspace\{1cm\}\n".
      "\t\t\{\\Large $use_dept\}\n\n".
      "\t\t\{\\Large $use_uni\}\n\n".
      "\t\t\{\\Large $use_city\}\n\n".
      "\t\t\{\\normalsize $use_email\}\n\n".
      "\t\t\\vspace\{1cm\}\n".
      "\t\t\{\\normalsize \\tt $use_online\}\n\n".
      "\t\\end\{center\}\n"
    );
  } else {
    &main::master_write( "% Insert PDF-Title here ...\n" );
    &main::master_write(
      "\t\\vspace\{3cm\}\n".
      "\t\\begin\{center\}\n".
      "\t\t\{\\huge $use_title\}\n\n".
      "\t\t\\vspace\{1.5cm\}\n".
      "\t\t\{\\Large $use_author\}\n\n".
      "\t\t\\vspace\{1cm\}\n".
      "\t\t\{\\Large $use_dept\}\n\n".
      "\t\t\{\\Large $use_uni\}\n\n".
      "\t\t\{\\Large $use_city\}\n\n".
      "\t\t\{\\normalsize $use_email\}\n\n".
      "\t\t\\vspace\{1cm\}\n".
      "\t\t\{\\normalsize \\tt $use_online\}\n\n".
      "\t\\end\{center\}\n"
    );
  }
}

1;
