#
# @command RAWHTML
# @title   Einbinden von HTML-Code
# @index   
# @syntax  \#RAWHTML\{\dq html-tags\dq\} \\\\ {\rm oder} \\\\ \#RAWHTML\{begin\}\\\qquad HTML-Code\\\#RAWHTML\{end\}
# @desc    Der {\tt \#RAWHTML}-Befehl erlaubt es, HTML-Befehle direkt in das Dokument einzuf"ugen.
#          In der \LaTeXe-Version wird er ignoriert. F"ur kleine HTML-Codeteile kann der HTML-Code
#          direkt im {\tt \#RAWHTML}-Befehl angegeben werden, bei l"angeren Codes sollte man die
#          {\tt \#RAWHTML}-Umgebung benutzen (s. Beispiel).
# @example_begin
#  #RAWHTML{begin}
#     <CENTER><P>
#        <IMG SRC="../picture/test.jpg" BORDER=0>
#     </CENTER>
#  #RAWHTML{end}
# @example_end
#
# @end

# Mas2TeX-Befehle definieren
$MANAGEFUNKTION->{RAWHTML}   = \&raw_work;

sub raw_work {
  package rawhtml;

  my ($html_tag);
  my @TAGS = @_;
  shift @TAGS;
  $html_tag = shift @TAGS;      # uebergebener HTML-Tag
  
  # RAWHTML-Umgebung starten
  if ($html_tag =~ /^(\s*)begin(\s*)/) {
    if (($main::OUTPUTFORMAT eq "latex") || ($main::OUTPUTFORMAT eq "pdf")) {
      # Alle nachfolgende Zeilen nicht in die Output-Datei schreiben
      $main::TEXFILE_WRITE_OK="";
    } else {
      print main::OUT "% *** Master-Generator - RAWHTML-Umgebung ***\n";
      print main::OUT "\\begin{rawhtml}\n";
    }

  # RAWHTML-Umgebung beenden
  } elsif ($html_tag =~ /^(\s*)end(\s*)/) {
    if (($main::OUTPUTFORMAT eq "latex") || ($main::OUTPUTFORMAT eq "pdf")) {
      # Alle nachfolgende Zeilen in die Output-Datei schreiben
      $main::TEXFILE_WRITE_OK="1";
    } else {
      print main::OUT  "\\end{rawhtml}\n";
    }

  # HTML-Befehle direkt einbinden
  } elsif ($main::OUTPUTFORMAT eq "html") {
    if ($html_tag) {
      print main::OUT "% BEGIN : RAWHTML\n";
      print main::OUT "\\begin{rawhtml}\n";
      print main::OUT "\t",$html_tag,"\n";
      foreach $html_tag (@TAGS) {    
        print main::OUT "\t",$html_tag,"\n";
      }
      print main::OUT  "\\end{rawhtml}\n";
    }
  }
  return;
}

1;
